@@ -28,24 +28,29 @@
 #define COMPONENT "ARP"
 
 
-#define T_REPLY	 2000000  /* 2 sec */
+#define T_REPLY	 1000000  /* 1 sec */
 #define R_REPLY	       5  /* 5 retries */
 #define T_REVAL	30000000  /* 30 sec - make much larger (if srv ...) @@@ */
+#define R_REVAL	       0  /* no retries */
+#define T_RETRY	 1000000  /* 1 sec */
+#define R_RETRY	      18  /* enough try keep trying during a long weekend */
 
 
-static void discard_vcc(VCC *vcc,int timeout)
+#define ENTER_TIMER_HANDLER(entry) entry->timer = NULL
+#define START_TIMER(entry,tmr) ({ \
+  if (entry->timer) stop_timer(entry->timer); \
+  entry->timeout = T_##tmr; \
+  entry->timer = start_timer(entry->timeout,timer_expiration,entry); \
+  entry->retries = R_##tmr; })
+#define STOP_TIMER(entry) ({ \
+  if (entry->timer) { \
+    stop_timer(entry->timer); \
+    entry->timer = NULL; \
+  } })
+
+
+static void discard_vcc(VCC *vcc)
 {
-    switch (vcc->state) {
-	case vs_valid:
-	    if (vcc->flags & ATF_PERM) break;
-	    /* fall trough */
-	case vs_discover:
-	case vs_reval:
-	    if (!timeout) stop_timer(vcc->timer);
-	    break;
-	default:
-	    break;
-    }
     if (do_close(vcc->fd))
 	diag(COMPONENT,DIAG_ERROR,"close: %s",strerror(errno));
     Q_REMOVE(vcc->entry->vccs,vcc);
@@ -53,16 +58,14 @@ static void discard_vcc(VCC *vcc,int timeout)
 }
 
 
-static void discard_entry(ENTRY *entry,int timeout)
+static void discard_entry(ENTRY *entry)
 {
     VCC *vcc,*next;
 
-    if (!timeout && entry->state != as_res_wait && entry->state != as_unknown)
-	if (entry->state != as_valid || !(entry->flags & ATF_PERM))
-	    stop_timer(entry->timer);
+    STOP_TIMER(entry);
     for (vcc = entry->vccs; vcc; vcc = next) {
 	next = vcc->next;
-	discard_vcc(vcc,0);
+	discard_vcc(vcc);
     }
     if (entry == entry->itf->arp_srv) entry->itf->arp_srv = NULL;
     Q_REMOVE(entry->itf->table,entry);
@@ -71,11 +74,48 @@ static void discard_entry(ENTRY *entry,int timeout)
 }
 
 
+/*
+ * Returns:
+ *  <0  there's no ARP server
+ *   0  ARP server connection establishment in progress
+ *  >0  ready to send ARP_REQUEST
+ */
+
+
+static int want_arp_srv(ITF *itf)
+{
+    VCC *vcc;
+    int fd;
+
+    if (!itf->arp_srv) return -1;
+    for (vcc = itf->arp_srv->vccs; vcc; vcc = vcc->next)
+	if (!vcc->connecting) return 1;
+    if (itf->arp_srv->vccs) return 0;
+    if ((fd = connect_vcc((struct sockaddr *) itf->arp_srv->addr)) < 0)
+	return 0;
+    vcc = alloc_t(VCC);
+    vcc->connecting = 1;
+    vcc->fd = fd;
+    vcc->entry = itf->arp_srv;
+    Q_INSERT_HEAD(itf->arp_srv->vccs,vcc);
+    return 0;
+}
+
+
 static void vcc_detach(ENTRY *entry)
 {
-    if (entry->state != as_res_wait && entry->state != as_unknown) return;
-    if (entry->flags & ATF_PERM) return;
-    discard_entry(entry,0);
+    ENTRY *walk;
+
+    if (entry->itf && entry->itf->arp_srv == entry)
+	for (walk = entry->itf->table; walk; walk = walk->next) {
+	    if (walk->state == as_resolv) break;
+	if (walk) {
+	    (void) want_arp_srv(entry->itf);
+	    return;
+	}
+    }
+    if (!entry->vccs && entry->state == as_invalid && !(entry->flags &
+      ATF_PERM)) discard_entry(entry);
 }
 
 
@@ -85,7 +125,7 @@ static void put_ip(unsigned char **here,unsigned long ip,unsigned char *len)
 	*len = 0;
 	return;
     }
-    ip = htonl(ip);
+    /*ip = htonl(ip); - no keep 'em in network byte order */
     memcpy(*here,&ip,4);
     *len = 4;
     (*here) += 4;
@@ -156,7 +196,7 @@ static void arp_request(ITF *itf,unsigned long ip)
 	return;
     }
     for (vcc = itf->arp_srv->vccs; vcc; vcc = vcc->next)
-	if (vcc->state != vs_connecting) break;
+	if (!vcc->connecting) break;
     if (!vcc) {
 	diag(COMPONENT,DIAG_ERROR,"ARP server has no usable VCC");
 	return;
@@ -165,15 +205,26 @@ static void arp_request(ITF *itf,unsigned long ip)
 }
 
 
-static void inarp_requests(VCC *vcc)
+static void inarp_for_itf(int fd,ITF *itf)
 {
+    if (itf->local_ip) {
+	diag(COMPONENT,DIAG_DEBUG,"  on itf %d",itf->number);
+	send_arp(fd,ARPOP_InREQUEST,itf->local_ip,&itf->local_addr,0,NULL);
+    }
+}
+
+
+static void inarp_request(ENTRY *entry)
+{
+    VCC *vcc;
     ITF *itf;
 
     diag(COMPONENT,DIAG_DEBUG,"sending InARP request");
-    for (itf = itfs; itf; itf = itf->next)
-	if (itf->local_ip)
-	    send_arp(vcc->fd,ARPOP_InREQUEST,itf->local_ip,&itf->local_addr,0,
-	      NULL);
+    for (vcc = entry->vccs; vcc; vcc = vcc->next)
+	if (!vcc->connecting)
+	    if (entry->itf) inarp_for_itf(vcc->fd,entry->itf);
+	    else for (itf = itfs; itf; itf = itf->next)
+		    inarp_for_itf(vcc->fd,itf);
 }
 
 
@@ -200,149 +251,82 @@ static void arp_reply(VCC *vcc,unsigned long src_ip,
 
 static void inarp_reply(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 {
+    ITF *itf;
+
     diag(COMPONENT,DIAG_DEBUG,"sending InARP reply");
-    send_arp(vcc->fd,ARPOP_InREPLY,itfs->local_ip,&itfs->local_addr,ip,addr);
-}
-
-
-static void timeout_vcc(void *user)
-{
-    VCC *vcc;
-    ENTRY *entry;
-
-    diag(COMPONENT,DIAG_DEBUG,"VCC TIMEOUT");
-    vcc = user;
-    switch (vcc->state) {
-	case vs_discover:
-	    if (!(vcc->flags & ATF_PERM) && !(vcc->retries)) {
-		discard_vcc(vcc,1);
-		if (vcc->entry) vcc_detach(vcc->entry);
-		break;
-	    }
-	    if (!(vcc->flags & ATF_PERM)) vcc->retries--;
-	    inarp_requests(vcc);
-	    vcc->timer = start_timer(T_REPLY,timeout_vcc,vcc);
-	    break;
-	case vs_valid:
-	    vcc->state = vs_reval;
-	    inarp_requests(vcc);
-	    vcc->timer = start_timer(T_REPLY,timeout_vcc,vcc);
-	    vcc->retries = R_REPLY;
-	    break;
-	case vs_reval:
-	    if (!vcc->retries--) {
-		vcc->entry->ip = 0;
-		if (set_ip(vcc->fd,0) < 0) {
-		    diag(COMPONENT,DIAG_ERROR,"can't set IP to 0");
-		    entry = vcc->entry;
-		    discard_vcc(vcc,1);
-		    vcc_detach(entry);
-		    break;
-		}
-		vcc->retries = R_REPLY;
-		vcc->state = vs_discover;
-	    }
-	    inarp_requests(vcc);
-	    vcc->timer = start_timer(T_REPLY,timeout_vcc,vcc);
-	    break;
-	default:
-	    diag(COMPONENT,DIAG_FATAL,"invalid VCC state %d",vcc->state);
-    }
-}
-
-
-/*
- * Returns:
- *  <0  there's no ARP server
- *   0  ARP server connection establishment in progress
- *  >0  ready to send ARP_REQUEST
- */
-
-
-static int want_arp_srv(ITF *itf)
-{
-    VCC *vcc;
-    int fd;
-
-    if (!itf->arp_srv) return -1;
-    for (vcc = itf->arp_srv->vccs; vcc; vcc = vcc->next)
-	if (vcc->state != vs_connecting) return 1;
-    if (itf->arp_srv->vccs) return 0;
-    if ((fd = connect_vcc((struct sockaddr *) &itf->local_addr,
-      (struct sockaddr *) itf->arp_srv->addr)) < 0)
-	return 0;
-    vcc = alloc_t(VCC);
-    vcc->state = vs_connecting;
-    vcc->fd = fd;
-    vcc->flags = itf->arp_srv->flags;
-    vcc->entry = itf->arp_srv;
-    Q_INSERT_HEAD(itf->arp_srv->vccs,vcc);
-    return 0;
-}
-
-
-static void timeout_entry(void *user);
-
-
-static void validate_entry(ENTRY *entry)
-{
-    int result;
-
-    if ((result = want_arp_srv(entry->itf)) < 0) {
-	/* @@@ try to revalidate by sending InARP */
-	return;
-    }
-    if (!result) {
-	if (entry->state == as_unknown) entry->state = as_res_wait;
+    itf = lookup_itf_by_ip(ip);
+    if (!itf) {
+	diag(COMPONENT,DIAG_ERROR,"InARP request is unroutable");
 	return;
     }
+    send_arp(vcc->fd,ARPOP_InREPLY,itf->local_ip,&itf->local_addr,ip,addr);
+}
+
+
+static void timer_expiration(void *user);
+
+
+static void revalidate(ENTRY *entry)
+{
+    entry->state = as_resolv;
+    if (!want_arp_srv(entry->itf)) return;
+    arp_request(entry->itf,entry->ip);
+    START_TIMER(entry,REPLY);
+}
+
+
+static void timer(ENTRY *entry)
+{
     switch (entry->state) {
-	case as_res_wait:
-	    diag(COMPONENT,DIAG_FATAL,"as_res_wait with ARP server up");
 	case as_resolv:
-	    return; /* waiting for result ... */
-	case as_unknown:
-	    entry->state = as_resolv;
 	    arp_request(entry->itf,entry->ip);
-	    entry->timer = start_timer(T_REPLY,timeout_entry,entry);
-	    entry->retries = R_REPLY;
-	    return;
-	case as_valid:
-	    return; /* race ... */
+	    break;
+	case as_invalid:
+	    inarp_request(entry);
+	    break;
 	default:
-	    diag(COMPONENT,DIAG_FATAL,"bad state %d",entry->state);
+	    diag(COMPONENT,DIAG_FATAL,"timer in state %d",entry->state);
     }
 }
 
 
-static void timeout_entry(void *user)
+static void timeout(ENTRY *entry)
 {
-    ENTRY *entry;
-    unsigned char *ipp;
-
-    entry = user;
-    ipp = (unsigned char *) &entry->ip;
-    diag(COMPONENT,DIAG_DEBUG,"TIMEOUT (entry %d.%d.%d.%d)",ipp[0],ipp[1],
-      ipp[2],ipp[3]);
+    VCC *vcc;
+    entry->timer = NULL;
     switch (entry->state) {
 	case as_resolv:
-	    if (entry->retries) {
-		entry->retries--;
-		arp_request(entry->itf,entry->ip);
-		entry->timer = start_timer(T_REPLY,timeout_entry,entry);
+	    if (entry->flags & ATF_ARPSRV && !entry->vccs) {
+		if (entry->itf) want_arp_srv(entry->itf);
 		break;
 	    }
-	    if (!entry->vccs && !(entry->flags & ATF_PERM))
-		discard_entry(entry,1);
-	    else entry->state = as_unknown;
+	    if (!entry->vccs && !(entry->flags & (ATF_PERM | ATF_ARPSRV)))
+		discard_entry(entry);
+	    else entry->state = as_invalid;
 	    break;
 	case as_valid:
-	    if (!entry->vccs && !(entry->flags & ATF_PERM)) {
-		discard_entry(entry,1);
+	    if (!entry->vccs && !(entry->flags & (ATF_PERM | ATF_ARPSRV)) &&
+	      entry->itf->arp_srv) {
+		discard_entry(entry);
 		break;
 	    }
-	    entry->state = as_unknown; /* avoid as_valid race */
-	    validate_entry(entry);
+	    for (vcc = entry->vccs; vcc; vcc = vcc->next)
+		if (set_ip(vcc->fd,entry->ip) < 0)
+		    diag(COMPONENT,DIAG_ERROR,"set_ip(0): %s",strerror(errno));
+	    if (entry->svc && entry->itf->arp_srv) revalidate(entry);
+	    else {
+		inarp_request(entry);
+		START_TIMER(entry,REPLY);
+		entry->state = as_invalid;
+	    }
+	    break;
+	case as_invalid:
+	    if (!entry->svc) {
+		inarp_request(entry);
+		START_TIMER(entry,REPLY);
+	    }
+	    else if ((!entry->itf || !entry->itf->arp_srv) &&
+		  !(entry->flags & ATF_PERM)) discard_entry(entry);
 	    break;
 	default:
 	    diag(COMPONENT,DIAG_FATAL,"timed out in state %s",
@@ -351,130 +335,103 @@ static void timeout_entry(void *user)
 }
 
 
-static void learn_vcc(VCC *vcc,unsigned long ip)
+static void timer_expiration(void *user)
 {
-    ITF *itf;
     ENTRY *entry;
+    unsigned char *ipp;
 
-    if (!ip) return;
-    for (itf = itfs; itf; itf = itf->next) {
-	entry = lookup_ip(itf,ip);
-	if (entry) break;
+    entry = user;
+    ipp = (unsigned char *) &entry->ip;
+    diag(COMPONENT,DIAG_DEBUG,"TIMEOUT (entry %d.%d.%d.%d), %d retries",ipp[0],
+      ipp[1],ipp[2],ipp[3],entry->retries);
+    if (!entry->retries) {
+	entry->timer = NULL;
+	timeout(entry);
     }
-    if (!vcc->entry) {
-	if (!entry) {
-	    entry = alloc_t(ENTRY);
-	    entry->state = as_unknown;
-	    entry->ip = ip;
-	    entry->addr = NULL;
-	    entry->flags = 0;
-	    entry->itf = itfs; /* lookup one ... @@@ */
-	    Q_INSERT_HEAD(entry->itf->table,entry);
-	}
-	Q_INSERT_HEAD(entry->vccs,vcc);
-	vcc->entry = entry;
-    }
-    else if (!entry) (entry = vcc->entry)->ip = ip;
-	else if (entry != vcc->entry) {
-		Q_REMOVE(vcc->entry->vccs,vcc);
-		vcc_detach(vcc->entry);
-		vcc->entry = entry;
-		Q_INSERT_HEAD(entry->vccs,vcc);
-	    }
-    switch (vcc->state) {
-	case vs_discover:
-	case vs_svc:
-	    if (set_ip(vcc->fd,ip) < 0) {
-		diag(COMPONENT,DIAG_ERROR,"can't set IP");
-		discard_vcc(vcc,0);
-		vcc_detach(entry);
-		break;
-	    }
-	    if (vcc->state == vs_svc) break;
-	    /* fall through */
-	case vs_valid:
-	case vs_reval:
-	    stop_timer(vcc->timer);
-	    vcc->timer = start_timer(T_REVAL,timeout_vcc,vcc);
-	    break;
-	default:
-	    diag(COMPONENT,DIAG_FATAL,"invalid VCC state %d",vcc->state);
+    else {
+	entry->retries--;
+	entry->timeout *= 2;
+	start_timer(entry->timeout,timer_expiration,entry);
+	timer(entry);
     }
 }
 
 
-static void learn_addr(unsigned long ip,struct sockaddr_atmsvc *addr)
+static void connect_me(ENTRY *entry)
 {
-    ITF *itf;
-    ENTRY *by_ip,*by_addr;
-    VCC *vcc,*next,*last;
+    VCC *vcc;
     int fd;
 
-    if (!ip || !addr) return;
-    by_ip = NULL;
-    if (!by_ip) /* very very ugly */
-	for (itf = itfs; itf; itf = itf->next) {
-	    by_ip = lookup_ip(itf,ip);
-	    if (by_ip) break;
-	}
-    if (by_ip) by_addr = lookup_addr(by_ip->itf,addr);
-    else {
-	for (itf = itfs; itf; itf = itf->next) {
-	    by_ip = lookup_addr(itf,addr);
-	    if (by_ip) break;
-	}
-	by_addr = by_ip;
+    if ((fd = connect_vcc((struct sockaddr *) entry->addr)) < 0) return;
+    vcc = alloc_t(VCC);
+    vcc->connecting = 1;
+    vcc->fd = fd;
+    vcc->entry = entry;
+    Q_INSERT_HEAD(entry->vccs,vcc);
+}
+
+
+static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
+{
+    ENTRY *entry;
+    ITF *itf;
+    VCC *walk;
+    unsigned char *ipp;
+
+    if (!ip) return;
+    ipp = (unsigned char *) &ip;
+    itf = lookup_itf_by_ip(ip);
+    if (!itf) {
+	diag(COMPONENT,DIAG_ERROR,"got unroutable IP address %d.%d.%d.%d",
+	  ipp[0],ipp[1],ipp[2],ipp[3]);
+	return;
     }
-    if (by_ip && by_addr) {
-	if (by_ip != by_addr) {
-	    by_addr->ip = ip;
-	    last = NULL;
-	    for (vcc = by_addr->vccs; vcc; vcc = vcc->next) last = vcc;
-	    if (!last) by_addr->vccs = by_ip->vccs;
-	    else {
-		last->next = by_ip->vccs;
-		if (by_ip->vccs) by_ip->vccs->prev = last;
-	    }
-	    by_addr->flags |= by_ip->flags; /* keep ATF_ARPSRV and such */
-	    by_ip->vccs = NULL;
-	    discard_entry(by_ip,0);
-	    by_ip = by_addr;
+    entry = lookup_ip(itf,ip);
+    if (entry && vcc && (!vcc->entry || entry != vcc->entry)) {
+	diag(COMPONENT,DIAG_DEBUG,"collision on %d.%d.%d.%d",ipp[0],ipp[1],
+	  ipp[2],ipp[3]);
+	return;
+    }
+    if (!entry && !(entry = vcc->entry)) {
+	entry = alloc_t(ENTRY);
+	entry->svc = 1;
+	entry->state = as_none;
+	entry->addr = NULL;
+	entry->flags = 0;
+	entry->timer = NULL;
+	entry->vccs = NULL;
+	entry->itf = NULL;
+	if (vcc) {
+	    vcc->entry = entry;
+	    Q_INSERT_HEAD(entry->vccs,vcc);
 	}
     }
-    else {
-	if (!by_ip) {
-	    by_ip = alloc_t(ENTRY);
-	    by_ip->state = as_unknown; /* has no timer */
-	    by_ip->flags = 0;
-	    by_ip->vccs = NULL;
-	    by_ip->itf = itfs; /* @@@ */
-	}
-	by_ip->ip = ip;
-	by_ip->addr = alloc_t(struct sockaddr_atmsvc); /* @@@ fixme */
-	*by_ip->addr = *addr;
+    STOP_TIMER(entry);
+    entry->ip = ip;
+    if (!entry->itf) {
+	entry->itf = itf;
+	Q_INSERT_HEAD(itf->table,entry);
     }
-    if (by_ip->state == as_resolv || (by_ip->state == as_valid &&
-      !(by_ip->flags & ATF_PERM))) stop_timer(by_ip->timer);
-    by_ip->state = as_valid;
-    if (ip)
-	for (vcc = by_ip->vccs; vcc; vcc = next) {
-	    next = vcc->next;
-	    if (set_ip(vcc->fd,ip) < 0) {
-		diag(COMPONENT,DIAG_ERROR,"set_ip failed");
-		Q_REMOVE(by_ip->vccs,vcc);
-	    }
-	}
-    if (!by_ip->vccs && (fd = connect_vcc(by_ip->itf ? (struct sockaddr *)
-      &by_ip->itf->local_addr : NULL,(struct sockaddr *) addr)) >= 0) {
-	vcc = alloc_t(VCC);
-	vcc->state = vs_connecting;
-	vcc->fd = fd;
-	vcc->flags = by_ip->flags;
-	vcc->entry = by_ip;
-	Q_INSERT_HEAD(by_ip->vccs,vcc);
+    if (entry->itf != itf)
+	diag(COMPONENT,DIAG_ERROR,"interesting, interface has changed ... "
+	  "(%d -> %d)",entry->itf->number,itf->number);
+    if (!entry->addr) entry->addr = alloc(sizeof(*addr));
+    *entry->addr = *addr;
+    /* need to revise this policy if we get user-settable QOS @@@ */
+    memset(&entry->addr->sas_txtp,0,sizeof(entry->addr->sas_txtp));
+    entry->addr->sas_txtp.class = ATM_UBR;
+    entry->addr->sas_txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
+    entry->addr->sas_rxtp = entry->addr->sas_txtp;
+    if (entry->state != as_valid) {
+	for (walk = entry->vccs; walk; walk = walk->next)
+	    if (!walk->connecting)
+		if (set_ip(walk->fd,ip) < 0)
+		    diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
+	if (!entry->vccs && itf->arp_srv) connect_me(entry);
     }
-    if (by_ip->state != as_valid || !(by_ip->flags & ATF_PERM))
-	by_ip->timer = start_timer(T_REVAL,timeout_entry,by_ip);
+    if ((entry->flags & ATF_ARPSRV) || !(entry->flags & ATF_PERM))
+	START_TIMER(entry,REVAL);
+    entry->state = as_valid;
 }
 
 
@@ -484,13 +441,13 @@ static void learn_nak(unsigned long ip)
     ENTRY *entry;
 
     if (!ip) return;
-    for (itf = itfs; itf; itf = itf->next) { /* ugly ... */
-	entry = lookup_ip(itf,ip);
-	if (entry) break;
-    }
-    if (!entry) return;
+    itf = lookup_itf_by_ip(ip);
+    if (!itf) return;
+    entry = lookup_ip(itf,ip);
+    if (!entry || entry->state != as_resolv) return;
     if (entry->flags & ATF_PERM) return;
-    discard_entry(entry,0);
+    if (entry->vccs || (entry->flags & ATF_PERM)) entry->state = as_invalid;
+    else discard_entry(entry);
 }
 
 
@@ -506,9 +463,24 @@ void need_ip(int itf_num,unsigned long ip)
 	diag(COMPONENT,DIAG_ERROR,"itf %d not found",itf_num);
 	return;
     }
-    if (!(entry = lookup_ip(itf,ip))) {
+    if (!itf->arp_srv) return; /* bad luck - no ARP server */
+    entry = lookup_ip(itf,ip);
+    if (entry)
+	switch (entry->state) {
+	    case as_resolv:
+		return; /* somebody else is already taking care of that */
+	    case as_valid:
+		if (!entry->vccs) connect_me(entry);
+		return;
+	    case as_invalid:
+		if (entry->svc && entry->itf && entry->itf->arp_srv) break;
+		return;
+	    default:
+		diag(COMPONENT,DIAG_FATAL,"bad state %d",entry->state);
+	}
+    else {
 	entry = alloc_t(ENTRY);
-	entry->state = as_unknown; /* force revalidation */
+	entry->svc = 1;
 	entry->ip = ip;
 	entry->addr = NULL;
 	entry->flags = 0;
@@ -516,7 +488,7 @@ void need_ip(int itf_num,unsigned long ip)
 	entry->itf = itf;
 	Q_INSERT_HEAD(itf->table,entry);
     }
-    validate_entry(entry);
+    revalidate(entry);
 }
 
 
@@ -556,7 +528,9 @@ static void set_addr(struct sockaddr_atmsvc *addr,void *num,void *sub,
 static unsigned long get_ip(unsigned char *ptr)
 {
     if (!ptr) return 0;
-    return (ptr[0] << 24) | (ptr[1] << 16) | (ptr[2] << 8) | ptr[3];
+    /* awkward, but this way we avoid bus errors on architectures that
+       don't support mis-aligned accesses */
+    return htonl((ptr[0] << 24) | (ptr[1] << 16) | (ptr[2] << 8) | ptr[3]);
 }
 
 
@@ -621,13 +595,10 @@ void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len)
     switch (ntohs(hdr->ar_op)) {
 	case ARPOP_REQUEST:
 	    diag(COMPONENT,DIAG_DEBUG,"got ARP_REQ");
-	    learn_vcc(vcc,src_ip);
-	    learn_addr(src_ip,&source);
+	    learn(vcc,src_ip,&source);
 	    entry = NULL;
-	    for (itf = itfs; itf; itf = itf->next) {
-		entry = lookup_ip(itf,tgt_ip);
-		if (entry) break;
-	    }
+	    itf = lookup_itf_by_ip(tgt_ip);
+	    entry = itf ? lookup_ip(itf,tgt_ip) : NULL;
 	    if (!entry) arp_nak(vcc,tgt_ip,src_ip,&source);
 	    else arp_reply(vcc,tgt_ip,entry->addr,src_ip,&source);
 	    break;
@@ -637,18 +608,16 @@ void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len)
 		diag(COMPONENT,DIAG_ERROR,"got ARP response from charlatan");
 		break;
 	    }
-	    learn_addr(src_ip,&source); /* entry != NULL ? @@@ */
+	    learn(NULL,src_ip,&source);
 	    break;
 	case ARPOP_InREQUEST:
 	    diag(COMPONENT,DIAG_DEBUG,"got InARP_REQ");
-	    learn_vcc(vcc,src_ip);
-	    learn_addr(src_ip,&source);
+	    learn(vcc,src_ip,&source);
 	    inarp_reply(vcc,src_ip,&source);
 	    break;
 	case ARPOP_InREPLY:
 	    diag(COMPONENT,DIAG_DEBUG,"got InARP_REP");
-	    learn_vcc(vcc,src_ip);
-	    learn_addr(src_ip,&source);
+	    learn(vcc,src_ip,&source);
 	    break;
 	case ARPOP_NAK:
 	    diag(COMPONENT,DIAG_DEBUG,"got ARP_NAK");
@@ -673,64 +642,49 @@ static int ioctl_set_pvc(ITF *itf,unsigned long ip,
     int fd,result;
 
     if (lookup_ip(itf,ip)) return -EEXIST; 
-    if ((fd = connect_vcc(NULL,(struct sockaddr *) addr)) < 0) return fd;
-    if (ip)
-	if ((result = set_ip(fd,ip)) < 0) return result;
+    if ((fd = connect_vcc((struct sockaddr *) addr)) < 0) return fd;
+    if ((result = set_ip(fd,ip)) < 0) return result;
     entry = alloc_t(ENTRY);
-    entry->state = as_unknown;
+    entry->state = as_valid;
+    entry->svc = 0;
     entry->ip = ip;
     entry->addr = NULL;
+    entry->flags = flags;
     entry->vccs = NULL;
     entry->itf = itf;
     entry->vccs = NULL;
     vcc = alloc_t(VCC);
-    vcc->state = vs_valid;
+    vcc->connecting = 0;
     vcc->fd = fd;
     vcc->entry = entry;
-    entry->flags = vcc->flags = flags;
-    if (ip) {
-	if (!(flags & ATF_PERM))
-	    vcc->timer = start_timer(T_REVAL,timeout_vcc,vcc);
-    }
-    else {
-	vcc->state = vs_discover;
-	inarp_requests(vcc);
-	vcc->timer = start_timer(T_REPLY,timeout_vcc,vcc);
-	vcc->retries = R_REPLY;
-    }
+    if (!(flags & ATF_PERM)) START_TIMER(entry,REVAL);
     Q_INSERT_HEAD(entry->vccs,vcc);
     Q_INSERT_HEAD(itf->table,entry);
     return 0;
 }
 
 
-static void set_local(ITF *itf,unsigned long ip,struct sockaddr_atmsvc *addr)
-{
-    itf->local_ip = ip;
-    itf->local_addr = *addr;
-}
-
-
 static int ioctl_set_svc(ITF *itf,unsigned long ip,
   struct sockaddr_atmsvc *addr,int flags)
 {
     ENTRY *entry;
 
-    if (flags & (ATF_LOCAL | ATF_ARPSRV)) flags |= ATF_PERM;
-    if (flags & ATF_LOCAL) set_local(itf,ip,addr);
+    if (flags & ATF_ARPSRV) flags |= ATF_PERM;
     if (lookup_ip(itf,ip)) return -EEXIST;
     entry = alloc_t(ENTRY);
     entry->state = as_valid;
+    entry->svc = 1;
     entry->ip = ip;
     entry->addr = alloc_t(struct sockaddr_atmsvc);
     *entry->addr = *addr;
+diag(COMPONENT,DIAG_DEBUG,"tx-sdu: %d",addr->sas_txtp.max_sdu);
     entry->flags = flags;
-    if (!(flags & ATF_PERM))
-	entry->timer = start_timer(T_REVAL,timeout_entry,entry);
+    if (!(flags & ATF_PERM) || (flags & ATF_ARPSRV)) START_TIMER(entry,REVAL);
     entry->vccs = NULL;
     entry->itf = itf;
     Q_INSERT_HEAD(itf->table,entry);
     if (flags & ATF_ARPSRV) {
+	entry->state = as_invalid;
 	itf->arp_srv = entry;
 	(void) want_arp_srv(itf);
     }
@@ -740,13 +694,27 @@ static int ioctl_set_svc(ITF *itf,unsigned long ip,
 
 static int ioctl_delete(ITF *itf,unsigned long ip)
 {
-    ENTRY *entry;
+    ENTRY *entry,*walk,*next;
 
     if (!(entry = lookup_ip(itf,ip))) {
 	diag(COMPONENT,DIAG_WARN,"ioctl_delete didn't find entry");
 	return -ENOENT;
     }
-    discard_entry(entry,0);
+    if ((entry->flags & ATF_ARPSRV) && entry->itf)
+	for (walk = entry->itf->table; walk; walk = next) {
+	    next = walk->next;
+	    if (walk != entry && walk->state == as_resolv)
+		if (!walk->vccs && !(walk->flags & ATF_PERM))
+		   /* PERM is rather unlikely here, since this would be a
+		      second ARP server (only ARP servers can go as_resolv if
+		      PERM), but we'll check for it anyway. */
+		    discard_entry(walk);
+		else {
+		    STOP_TIMER(walk);
+		    walk->state = as_invalid;
+		}
+	}
+    discard_entry(entry);
     return 0;
 }
 
@@ -758,6 +726,7 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
     unsigned long ip;
     unsigned char *ipp;
 
+    diag(COMPONENT,DIAG_DEBUG,"arp_ioctl 0x%08lx",cmd);
     if (req->arp_pa.sa_family != AF_INET) {
 	diag(COMPONENT,DIAG_ERROR,"SIOCSARP: bad PA AF 0x%x",
 	  req->arp_pa.sa_family);
@@ -809,8 +778,25 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
 
 void vcc_failed(VCC *vcc)
 {
+    ENTRY *entry,*next;
+
+    diag(COMPONENT,DIAG_DEBUG,"failed VCC 0x%08x",(unsigned long) vcc);
     Q_REMOVE(vcc->entry->vccs,vcc);
-    vcc_detach(vcc->entry);
+    if (!(vcc->entry->flags & ATF_ARPSRV) || vcc->entry->vccs != vcc ||
+      vcc->next) {
+	/* VCC is already closed */
+	vcc_detach(vcc->entry);
+	free(vcc);
+	return;
+    }
+    for (entry = vcc->entry->itf->table; entry; entry = next) {
+	next = entry->next;
+	if (entry != vcc->entry && entry->state == as_resolv)
+	    if (entry->vccs || (entry->flags & ATF_PERM))
+		entry->state = as_invalid;
+	    else discard_entry(vcc->entry);
+    }
+    START_TIMER(entry,RETRY);
     free(vcc);
 }
 
@@ -819,23 +805,19 @@ void vcc_connected(VCC *vcc)
 {
     ENTRY *entry;
 
-    diag(COMPONENT,DIAG_DEBUG,"connected entry 0x%08x",(unsigned long) entry);
-    if (vcc->state != vs_connecting)
+    diag(COMPONENT,DIAG_DEBUG,"connected VCC 0x%08x",(unsigned long) vcc);
+    if (!vcc->connecting)
 	diag(COMPONENT,DIAG_FATAL,"non-connecting VCC connected");
-    vcc->state = vs_svc;
-    if (set_ip(vcc->fd,vcc->entry->ip) < 0) {
-	diag(COMPONENT,DIAG_ERROR,"can't set IP");
-	vcc_failed(vcc);
-	return;
-    }
-    if (!(vcc->entry->flags & ATF_ARPSRV)) return;
-    for (entry = vcc->entry->itf->table; entry; entry = entry->next)
-	if (entry->state == as_res_wait) {
-	    entry->state = as_resolv;
-	    entry->timer = start_timer(T_REPLY,timeout_entry,entry);
-	    entry->retries = R_REPLY;
-	    arp_request(entry->itf,entry->ip);
-	}
+    vcc->connecting = 0;
+    if (vcc->entry->flags & ATF_ARPSRV)
+	for (entry = vcc->entry->itf->table; entry; entry = entry->next)
+	    if (entry->state == as_resolv) {
+		START_TIMER(entry,REPLY);
+		arp_request(entry->itf,entry->ip);
+	    }
+    if (vcc->entry->state != as_valid) inarp_request(vcc->entry);
+    else if (set_ip(vcc->fd,vcc->entry->ip) < 0)
+	    diag(COMPONENT,DIAG_ERROR,"can't set IP (%s)",strerror(errno));
 }
 
 
@@ -843,13 +825,24 @@ void disconnect_vcc(VCC *vcc)
 {
     ENTRY *entry;
 
+    diag(COMPONENT,DIAG_DEBUG,"disconnected VCC0x%08x",(unsigned long) vcc);
+    /* immediately try to bring it up again if this was the connection to the
+       ATMARP server and we still need it. Should delay this in case the ARP
+       server has a real problem. @@@ */
+    if (vcc->entry && vcc->entry->itf && (vcc->entry->flags & ATF_ARPSRV)) {
+	for (entry = vcc->entry->itf->table; entry; entry = entry->next)
+	   if (entry->state == as_resolv) break;
+	if (entry) want_arp_srv(vcc->entry->itf);
+    }
     entry = vcc->entry;
-    discard_vcc(vcc,0);
+    discard_vcc(vcc);
     if (entry) vcc_detach(entry);
 }
 
 
 void incoming_call(VCC *vcc)
 {
-    inarp_requests(vcc);
+    diag(COMPONENT,DIAG_DEBUG,"incoming VCC 0x%08x",(unsigned long) vcc);
+    START_TIMER(vcc->entry,REPLY);
+    inarp_request(vcc->entry);
 }
