@@ -42,7 +42,7 @@ int main(int argc,char **argv)
 
     set_verbosity(NULL,DIAG_INFO);
     background = 0;
-    while ((c = getopt(argc,argv,"bd")) != EOF)
+    while ((c = getopt(argc,argv,"bdnp")) != EOF)
 	switch (c) {
 	    case 'b':
 		background = 1;
@@ -51,6 +51,10 @@ int main(int argc,char **argv)
 		set_verbosity(NULL,DIAG_DEBUG);
 		debug = 1;
 		break;
+	    case 'n':
+		/* will enable NSAP matching @@@ */
+	    case 'p':
+		/* paranoid anti-firewall-tunneling mode @@@ */
 	    default:
 		usage(argv[0]);
 	}
