@@ -12,9 +12,12 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
+#include <net/if.h>
+#include <netinet/in.h>
 #include <linux/atm.h>
 #include <linux/atmarp.h>
 
+#include "atm.h"
 #include "atmd.h"
 
 #include "table.h"
@@ -29,7 +32,7 @@
 struct timeval now;
 
 
-static int kernel,incoming;
+static int kernel,incoming,inet;
 
 
 /* ----- kernel interface -------------------------------------------------- */
@@ -40,8 +43,10 @@ void open_all(void)
     struct sockaddr_atmsvc addr;
     struct atm_blli blli;
 
-    if ((kernel = socket(PF_ATMSVC,SOCK_DGRAM,ATM_ATMARP)) < 0)
+    if ((kernel = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0)
 	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
+    if (ioctl(kernel,ATMARPD_CTRL,0) < 0)
+	diag(COMPONENT,DIAG_FATAL,"ioctl ATMARPD_CTRL: %s",strerror(errno));
     if ((incoming = socket(PF_ATMSVC,SOCK_DGRAM,ATM_AAL5)) < 0)
 	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
     memset(&addr,0,sizeof(addr));
@@ -55,6 +60,8 @@ void open_all(void)
 	diag(COMPONENT,DIAG_FATAL,"bind: %s",strerror(errno));
     if (listen(incoming,5) < 0)
 	diag(COMPONENT,DIAG_FATAL,"listen: %s",strerror(errno));
+    if ((inet = socket(PF_INET,SOCK_DGRAM,0)) < 0)
+	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
 }
 
 
@@ -88,12 +95,15 @@ static void recv_kernel(void)
 	case act_need:
 	    need_ip(ctrl->itf_num,ctrl->arg);
 	    break;
-	case act_up:
+	case act_create:
 	    ctrl->type = act_complete;
-	    ctrl->arg = itf_up(ctrl->itf_num);
+	    ctrl->arg = itf_create(ctrl->itf_num);
 	    if (write(kernel,ctrl,sizeof(*ctrl)) < 0)
 		diag(COMPONENT,DIAG_ERROR,"write reply: %s",strerror(errno));
 	    break;
+	case act_up:
+	    itf_up(ctrl->itf_num);
+	    break;
 	case act_down:
 	    itf_down(ctrl->itf_num);
 	    break;
@@ -115,6 +125,7 @@ void close_all(void)
 {
     (void) close(incoming);
     (void) close(kernel); /* may get major complaints from the kernel ... */
+    (void) close(inet);
 }
 
 
@@ -163,8 +174,8 @@ static void recv_vcc(VCC *vcc)
 
 static void accept_new(void)
 {
+    char buffer[MAX_ATM_ADDR_LEN+1];
     struct sockaddr_atmsvc addr;
-    ITF *itf;
     ENTRY *entry;
     VCC *vcc;
     int fd,len;
@@ -180,8 +191,9 @@ static void accept_new(void)
         return;
     }
     vcc = alloc_t(VCC);
-    vcc->state = vs_svc;
+    vcc->connecting = 0;
     vcc->fd = fd;
+#if 0
     for (itf = itfs; itf; itf = itf->next) {
 	entry = lookup_addr(itf,&addr);
 	if (entry) {
@@ -191,23 +203,26 @@ static void accept_new(void)
 	    return;
 	}
     }
+#endif
     entry = alloc_t(ENTRY);
-    entry->state = as_unknown;
+    entry->state = as_invalid;
+    entry->svc = 1;
     entry->ip = 0;
     entry->addr = alloc_t(struct sockaddr_atmsvc);
     *entry->addr = addr;
     entry->flags = 0;
     entry->vccs = NULL;
-    entry->itf = itfs; /* grr, yet another routing decision @@@ */
-    vcc->flags = 0;
+    entry->itf = NULL;
     vcc->entry = entry;
     Q_INSERT_HEAD(entry->vccs,vcc);
-    Q_INSERT_HEAD(itfs->table,entry);
+    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
+      A2T_PRETTY) < 0) strcpy(buffer,"<atm2text error>");
+    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
     incoming_call(vcc);
 }
 
 
-int connect_vcc(struct sockaddr *local,struct sockaddr *remote)
+int connect_vcc(struct sockaddr *remote)
 {
     int fd,flags;
 
@@ -215,6 +230,7 @@ int connect_vcc(struct sockaddr *local,struct sockaddr *remote)
 	diag(COMPONENT,DIAG_ERROR,"socket: %s",strerror(errno));
 	return -errno;
     }
+#if 0 /* ilmid and sigd do that now for us */
     if (local) {
     if (debug) {
 	int i;
@@ -229,6 +245,7 @@ int connect_vcc(struct sockaddr *local,struct sockaddr *remote)
 	    return -errno;
 	}
 }
+#endif
     if ((flags = fcntl(fd,F_GETFL)) < 0) {
 	diag(COMPONENT,DIAG_ERROR,"fcntl F_GETFL: %s",strerror(errno));
 	return -errno;
@@ -246,9 +263,10 @@ int connect_vcc(struct sockaddr *local,struct sockaddr *remote)
 	blli.l3_proto = ATM_L3_NONE;
 	blli.next = NULL;
     }
+    /* PVC connect never blocks */
     if (connect(fd,remote,remote->sa_family == AF_ATMPVC ?
       sizeof(struct sockaddr_atmpvc) : sizeof(struct sockaddr_atmsvc)) < 0) {
-	diag(COMPONENT,DIAG_ERROR,"connect: %s",strerror(errno));
+	diag(COMPONENT,DIAG_ERROR,"[1]connect: %s",strerror(errno));
 	if (errno != EINPROGRESS) return -errno;
     }
     if (ioctl(fd,ATMARP_MKIP,0) < 0) {
@@ -273,14 +291,14 @@ static void complete_connect(VCC *vcc)
 {
     struct sockaddr_atmsvc dummy;
 
-    if (vcc->state != vs_connecting)
-	diag(COMPONENT,DIAG_FATAL,"conecting non-connecting VCC 0x%08x",
+    if (!vcc->connecting)
+	diag(COMPONENT,DIAG_FATAL,"connecting non-connecting VCC 0x%08x",
 	  (unsigned long) vcc);
     memset(&dummy,0,sizeof(dummy));
     if (!connect(vcc->fd,(struct sockaddr *) &dummy,sizeof(dummy)))
 	vcc_connected(vcc);
     else {
-	diag(COMPONENT,DIAG_INFO,"connect: %s",strerror(errno));
+	diag(COMPONENT,DIAG_INFO,"[2]connect: %s",strerror(errno));
 	(void) do_close(vcc->fd);
 	vcc_failed(vcc);
     }
@@ -304,8 +322,8 @@ void poll_loop(void)
 	for (itf = itfs; itf; itf = itf->next)
 	    for (entry = itf->table; entry; entry = entry->next)
 		for (vcc = entry->vccs; vcc; vcc = vcc->next) {
-		    if (vcc->state != vs_connecting) FD_SET(vcc->fd,&rset);
-		    else FD_SET(vcc->fd,&cset);
+		    if (vcc->connecting) FD_SET(vcc->fd,&cset);
+		    else FD_SET(vcc->fd,&rset);
 		    if (vcc->fd >= fds) fds = vcc->fd+1;
 		}
 	for (vcc = pending; vcc; vcc = vcc->next) {
@@ -353,3 +371,40 @@ void send_packet(int fd,void *data,int length)
 	diag(COMPONENT,DIAG_ERROR,"write: %s",strerror(errno));
     else diag(COMPONENT,DIAG_ERROR,"short write: %d < %d",wrote,length);
 }
+
+
+int ip_itf_info(int number,unsigned long *ip,unsigned long *netmask)
+{
+    struct ifreq req;
+    unsigned char *p;
+
+    sprintf(req.ifr_ifrn.ifrn_name,"atm%d",number);
+    if (ioctl(inet,SIOCGIFADDR,&req) < 0) {
+	diag(COMPONENT,DIAG_FATAL,"ioctl SIOCGIFADDR: %s",strerror(errno));
+	return -1;
+    }
+    *ip = ((struct sockaddr_in *) &req.ifr_ifru.ifru_addr)->sin_addr.s_addr;
+    if (ioctl(inet,SIOCGIFNETMASK,&req) < 0) {
+	diag(COMPONENT,DIAG_FATAL,"ioctl SIOCGIFNETMASK: %s",strerror(errno));
+	return -1;
+    }
+    *netmask = ((struct sockaddr_in *) &req.ifr_ifru.ifru_netmask)->
+      sin_addr.s_addr;
+    p = (unsigned char *) ip;
+printf("ip %d.%d.%d.%d",p[0],p[1],p[2],p[3]);
+    p = (unsigned char *) netmask;
+printf(" mask %d.%d.%d.%d\n",p[0],p[1],p[2],p[3]);
+    return 0;
+}
+
+
+int atm_itf_info(struct sockaddr_atmsvc *addr)
+{
+    struct atmif_sioc req;
+
+    req.number = 0; /* @@@ */
+    req.arg = addr;
+    if (ioctl(kernel,SIOCGIFATMADDR,&req) >= 0) return 0;
+    diag(COMPONENT,DIAG_FATAL,"ioctl SIOCGIFATMADDR: %s",strerror(errno));
+    return -1;
+}
