@@ -19,10 +19,7 @@
 #define COMPONENT "TABLE"
 
 
-const char *entry_state_name[] = { "NONE","RES_WAIT","RESOLV","UNKNOWN",
-  "VALID" };
-const char *vcc_state_name[] =  { "NONE","DISCOVER","REVAL","VALID",
-  "CONNECTING","SVC" };
+const char *entry_state_name[] = { "NONE","RESOLV","INVALID","VALID" };
 
 
 ENTRY *lookup_ip(const ITF *itf,unsigned long ip)
@@ -61,7 +58,7 @@ static void dump_vcc(VCC *vcc)
 	strcpy(buffer,"<getsocknam error>");
     }
     else {
-	if (debug) {
+	if (0 && debug) { /* @@@ */
 	    int i;
 
 	    for (i = 0; i < size; i++)
@@ -71,8 +68,8 @@ static void dump_vcc(VCC *vcc)
 	if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
 	  A2T_PRETTY) < 0) strcpy(buffer,"<atm2text error>");
     }
-    diag(COMPONENT,DIAG_DEBUG,"  state %s, %s, flags 0x%x",
-      vcc_state_name[vcc->state],buffer,vcc->flags);
+    diag(COMPONENT,DIAG_DEBUG,"  %s%s",buffer,vcc->connecting ? ", connecting"
+      : "");
 }
 
 
