@@ -11,35 +11,37 @@
 #include "atmd.h"
 
 
-typedef enum { vs_none,vs_discover,vs_reval,vs_valid,vs_connecting,vs_svc }
-  VCC_STATE;
-
 typedef struct _vcc {
-    VCC_STATE state;
+    int connecting;
     int fd;
-    int flags;
-    TIMER *timer;
-    int retries;
     struct _entry *entry;
     struct _vcc *prev,*next;
 } VCC;
 
-typedef enum { as_none,as_res_wait,as_resolv,as_unknown,as_valid } ADDR_STATE;
+typedef enum {
+    as_none,		/* invalid */
+    as_resolv,		/* waiting for resolver response */
+    as_invalid,		/* invalid, waiting for all VCs to be closed */
+    as_valid,		/* valid */
+} ADDR_STATE;
 
 typedef struct _entry {
     ADDR_STATE state;
+    int svc;
     unsigned long ip;
     struct sockaddr_atmsvc *addr; /* NULL if none */
     int flags;
-    TIMER *timer;
+    TIMER *timer; /* currently active timer or NULL */
+    int timeout; /* current interval - only necessary if using retries */
     int retries;
     struct _vcc *vccs;
     struct _itf *itf;
-    struct _entry *prev,*next;
+    struct _entry *prev,*next; /* undefined if itf == NULL */
 } ENTRY;
 
 typedef struct _itf {
     unsigned long local_ip; /* @@@ */
+    unsigned long netmask;
     struct sockaddr_atmsvc local_addr; /* @@@ */
     int number;
     ENTRY *table;
