@@ -26,8 +26,8 @@
 static void usage(const char *name)
 {
     fprintf(stderr,"usage: %s -c atmN\n",name);
-    fprintf(stderr,"%6s %s -s ip_addr itf.vpi.vci [perm] [null]\n","",name);
-    fprintf(stderr,"%6s %s -s ip_addr atm_addr [perm] [arpsrv]\n","",name);
+    fprintf(stderr,"%6s %s -s ip_addr [itf.]vpi.vci [temp] [null]\n","",name);
+    fprintf(stderr,"%6s %s -s ip_addr atm_addr [temp] [arpsrv]\n","",name);
     fprintf(stderr,"%6s %s -d ip_addr\n","",name);
     exit(1);
 }
@@ -112,10 +112,11 @@ int main(int argc,char **argv)
     if (op == SIOCSARP) {
 	req.arp_flags = 0;
 	for (i = optind+2; i < argc; i++)
-		if (!strcmp(argv[i],"perm")) req.arp_flags |= ATF_PERM;
+		if (!strcmp(argv[i],"term")) req.arp_flags |= ATF_PERM;
 		else if (!strcmp(argv[i],"null")) req.arp_flags |= ATF_NULL;
 		else if (!strcmp(argv[i],"arpsrv")) req.arp_flags |= ATF_ARPSRV;
 		else usage(argv[0]);
+	req.arp_flags ^= ATF_PERM;
 	if (text2atm(argv[optind+1],(struct sockaddr *) &req.arp_ha,
 	  sizeof(req.arp_ha),T2A_NAME) < 0) {
 	    fprintf(stderr,"%s: invalid ATM address\n",argv[i]);
