@@ -20,11 +20,16 @@
 #define A2T_PRETTY	 1	/* add syntactic sugar */
 #define A2T_NAME	 2	/* attempt name lookup */
 
+#define AXE_WILDCARD	 1	/* allow wildcard match */
+#define AXE_PRVOPT	 2	/* private part is optional */
+
 
 #define MAX_ATM_ADDR_LEN (2*ATM_ESA_LEN+ATM_E164_LEN+5)
 				/* 4 dots, 1 plus */
 
 int text2atm(const char *text,struct sockaddr *addr,int length,int flags);
 int atm2text(char *buffer,int length,const struct sockaddr *addr,int flags);
+int atm_equal(const struct sockaddr_atmsvc *a,const struct sockaddr_atmsvc *b,
+  int len,int flags);
 
 #endif
