@@ -29,8 +29,12 @@ int main(int argc,char **argv)
 	    perror("ioctl SIOCGIFATMADDR");
 	    return 1;
 	}
+	if (!addr.sas_family) {
+	    printf("No address\n");
+	    return 0;
+	}
 	if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
-	  A2T_PRETTY)) {
+	  A2T_PRETTY) < 0) {
 	    fprintf(stderr,"bad address (internal error)\n");
 	    return 1;
 	}
@@ -38,7 +42,7 @@ int main(int argc,char **argv)
 	return 0;
     }
     if (argc > 2 || text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),
-      T2A_SVC | T2A_NAME)) {
+      T2A_SVC | T2A_NAME) < 0) {
 	fprintf(stderr,"usage: %s [atm_address]\n",argv[0]);
 	return 1;
     }
