@@ -6,6 +6,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -84,7 +85,7 @@ int main(int argc,char **argv)
 	}
 	req.arg = &set;
 	if (ioctl(s,SONET_GETDIAG,&req) < 0)
-	    perror("ioctl SONET_GETDIAG");
+	    if (errno != EINVAL) perror("ioctl SONET_GETDIAG");
 	if (stats.section_bip != -1)
 	    printf("Section BIP errors: %10ld\n",stats.section_bip);
 	if (stats.line_bip != -1)
