@@ -488,8 +488,9 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		    pos += *pc;
 		    pc += 3;
 		    for (len = *pc++; len; len--)
-			if (*pc++ != value) pc += 2;
+			if (*pc != value && *pc != -1) pc += 3;
 			else {
+			    pc++;
 			    for (group = *pc++; group != -1; group =
 			      groups[group].parent)
 				q_set(dsc->group_present,group);
