@@ -27,7 +27,7 @@ extern void q_report(int severity,const char *msg,...);
 typedef struct {
     unsigned char *data;
     unsigned char *required;
-    unsigned char *item_present;
+    unsigned char *field_present;
     unsigned char *group_present;
     int *length;
     void *buffer;
@@ -47,5 +47,6 @@ void q_write(Q_DSC *dsc,int field,void *buf,int size);
 int q_present(Q_DSC *dsc,int field);
 unsigned long q_fetch(Q_DSC *dsc,int field);
 int q_read(Q_DSC *dsc,int field,void *buf,int size);
- 
+int q_length(Q_DSC *dsc,int field);
+
 #endif
