@@ -9,9 +9,12 @@
 #include "file.h"
 
 
-static void dump_required(ITEM *start,ITEM *leader,int group)
+static int unique;
+
+
+static void dump_required(FIELD *start,FIELD *leader,int group)
 {
-    ITEM *walk;
+    FIELD *walk;
 
     for (walk = start; walk; walk = walk->next)
 	if (!walk->value) {
@@ -19,7 +22,7 @@ static void dump_required(ITEM *start,ITEM *leader,int group)
 		to_c("static int required_%d[] = {\n",group);
 		leader->has_required = 1;
 	    }
-	    to_c("    %d, /* %s */\n",walk->item,walk->id);
+	    to_c("    %d, /* %s */\n",walk->field,walk->id);
 	}
 	else if (walk->value->type == vt_length)
 		dump_required(walk->value->block,leader,group);
@@ -28,9 +31,9 @@ static void dump_required(ITEM *start,ITEM *leader,int group)
 }
 
 
-static void find_required(ITEM *start)
+static void find_required(FIELD *start)
 {
-    ITEM *walk;
+    FIELD *walk;
     TAG *scan;
 
     for (walk = start; walk; walk = walk->next)
@@ -55,9 +58,9 @@ static void find_required(ITEM *start)
 }
 
 
-static void groups(ITEM *start,int group)
+static void groups(FIELD *start,int group)
 {
-    ITEM *walk;
+    FIELD *walk;
     TAG *scan;
 
     for (walk = start; walk; walk = walk->next) {
@@ -68,6 +71,7 @@ static void groups(ITEM *start,int group)
 		case vt_case:
 		case vt_multi:
 		    for (scan = walk->value->tags; scan; scan = scan->next) {
+			if (scan->id) to_test("    \"%s\",\n",scan->id);
 			if (scan->block && scan->block->has_required)
 			    to_c("    { %d, required_%d }, /* %d: %s */ \n",
 			      group,scan->group,scan->group,scan->block->id);
@@ -85,10 +89,11 @@ static void groups(ITEM *start,int group)
 }
 
 
-static void values(ITEM *start)
+static void values(FIELD *start)
 {
-    ITEM *walk;
+    FIELD *walk;
     TAG *scan;
+    VALUE_LIST *tag;
 
     for (walk = start; walk; walk = walk->next)
 	if (walk->value)
@@ -97,9 +102,17 @@ static void values(ITEM *start)
 		    break;
 		case vt_case:
 		    if (*walk->id != '_') {
-			to_c("static int values_%d[] = {\n",walk->item);
+			to_c("static int values_%d[] = { /* %s */\n",
+			  walk->field,walk->id);
 			for (scan = walk->value->tags; scan; scan = scan->next)
+			  {
 			    to_c("    %s, %d,\n",scan->value,scan->group);
+			    for (tag = scan->more; tag; tag = tag->next)
+				to_c("    %s, %d,\n",tag->value,scan->group);
+			    if (scan->deflt) to_c("    -2, %d,\n",scan->group);
+				/* could also skip while entry, but maybe we'll
+				   want to use the tags later */
+			}
 			to_c("    -1, -1\n};\n\n");
 		    }
 		    /* fall through */
@@ -116,16 +129,17 @@ static void values(ITEM *start)
 }
 
 
-static void items(ITEM *start,int group)
+static void fields(FIELD *start,int group)
 {
-    ITEM *walk;
+    FIELD *walk;
     TAG *scan;
+    VALUE_LIST *tag;
 
     for (walk = start; walk; walk = walk->next) {
 	if (*walk->id != '_')
 	    if (walk->value && walk->value->type == vt_case)
 		to_c("    { %d, %d, %d, values_%d, %d }, /* %s */\n",group,
-		  walk->pos,walk->size,walk->item,walk->var_len,walk->id);
+		  walk->pos,walk->size,walk->field,walk->var_len,walk->id);
 	    else to_c("    { %d, %d, %d, NULL, %d }, /* %s */\n",group,
 		  walk->pos,walk->size,walk->var_len,walk->id);
 	if (walk->value)
@@ -135,10 +149,18 @@ static void items(ITEM *start,int group)
 		case vt_case:
 		case vt_multi:
 		    for (scan = walk->value->tags; scan; scan = scan->next)
-			items(scan->block,scan->group);
+			fields(scan->block,scan->group);
+		    to_test("static int unique_%d[] = { /* %s */\n",unique++,
+		      walk->id);
+		    for (scan = walk->value->tags; scan; scan = scan->next) {
+			to_test("    %s,\n",scan->value);
+			for (tag = scan->more; tag; tag = tag->next)
+			    to_test("    %s,\n",tag->value);
+		    }
+		    to_test("    -1\n};\n\n");
 		    break;
 		case vt_length:
-		    items(walk->value->block,group);
+		    fields(walk->value->block,group);
 		    break;
 		default:
 		    abort();
@@ -147,11 +169,13 @@ static void items(ITEM *start,int group)
 }
 
 
-void second(ITEM *def)
+void second(FIELD *def)
 {
+    int i;
+
     def->has_required = 0;
     to_c("\n/*\n");
-    to_c(" * If a groups contains required fields, these are listed in the\n");
+    to_c(" * If a group contains required fields, these are listed in the\n");
     to_c(" * following arrays. Each list ends with -1. The variable names\n");
     to_c(" * contain the group number.\n */\n\n");
     dump_required(def,def,0);
@@ -161,15 +185,20 @@ void second(ITEM *def)
     to_c("static GROUP groups[] = {\n");
     if (def->has_required) to_c("    { -1, required_0 },\n");
     else to_c("    { -1, NULL },\n");
+    to_test("static const char *groups[] = {\n");
     groups(def,0);
+    to_test("    NULL\n};\n\n");
     to_c("};\n\n\n");
     to_c("/*\n * Named case selectors only have a limited set of valid\n");
     to_c(" * values. They are listed in the following arrays.\n */\n\n");
     values(def);
     to_c("\n/*\n * Various information about fields.\n */\n\n");
     to_c("typedef struct {\n    int parent;\n    int pos,size;\n");
-    to_c("    int *values;\n    int actual;\n} ITEM;\n\n");
-    to_c("static ITEM items[] = {\n");
-    items(def,0);
+    to_c("    int *values;\n    int actual;\n} FIELD;\n\n");
+    to_c("static FIELD fields[] = {\n");
+    fields(def,0);
     to_c("};\n\n");
+    to_test("static int *unique[] = {\n");
+    for (i = 0; i < unique; i++) to_test("    unique_%d,\n",i);
+    to_test("    NULL\n};\n\n");
 }
