@@ -138,7 +138,8 @@ static void parser(FIELD *start)
 		    code("%s%d%d%d%d/* %s */\n","OP_CASE",walk->jump,
 		      walk->pos,walk->size,count,walk->id);
 		    for (scan = walk->value->tags; scan; scan = scan->next) {
-			code("%s%d%s\n",scan->value,scan->group,"?");
+			code("%s%d%s\n",scan->deflt ? "-1" : scan->value,
+			  scan->group,"?");
 			scan->patch = pc-1;
 			for (tag = scan->more; tag; tag = tag->next) {
 			    code("%s%d%s\n",tag->value,scan->group,"?");
