@@ -10,6 +10,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/ioctl.h>
 #include <linux/atm.h>
 #include <linux/atmsvc.h>
 
@@ -36,17 +37,14 @@ static int kernel,signaling;
 
 static int open_kernel(void)
 {
-    struct sockaddr_atmsvc addr;
     int s;
 
     if ((s = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0) {
 	perror("socket");
 	return -1;
     }
-    memset(&addr,0,sizeof(addr));
-    addr.sas_family = AF_ATMSVC;
-    if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) { /* use connect? */
-	perror("bind");
+    if (ioctl(s,ATMSIGD_CTRL,0) < 0) {
+	perror("ioctl ATMSIGD_CTRL");
 	return -1;
     }
     return s;
@@ -97,8 +95,9 @@ void to_kernel(struct atmsvc_msg *msg)
     size = bllis2msg(0);
     for (walk = msg->remote.sas_addr.blli; walk; walk = walk->next)
 	size += sizeof(struct atm_blli);
-    diag(COMPONENT,DIAG_DEBUG,"TO KERNEL: %s (%d) for 0x%lx/0x%lx <%d>",
+    diag("KERNEL",DIAG_DEBUG,"TO KERNEL: %s (%d) for 0x%lx/0x%lx <%d>",
       as_name[msg->type],msg->reply,msg->vcc,msg->listen_vcc,size);
+	/* should be "IO" ... */
     wrote = write(kernel,msg,size);
     if (wrote == size) return;
     if (wrote < 0) {
@@ -124,7 +123,7 @@ static int open_signaling(int interface)
     struct sockaddr_atmpvc addr;
     int s;
 
-    if ((s= socket(PF_ATMPVC,SOCK_DGRAM,ATM_AAL5)) < 0) {
+    if ((s = socket(PF_ATMPVC,SOCK_DGRAM,ATM_AAL5)) < 0) {
 	perror("socket");
 	return -1;
     }
@@ -134,6 +133,7 @@ static int open_signaling(int interface)
     addr.sap_addr.vpi = signaling_vpi;
     addr.sap_addr.vci = signaling_vci;
     addr.sap_rxtp.class = addr.sap_txtp.class = ATM_UBR;
+    addr.sap_rxtp.max_sdu = addr.sap_txtp.max_sdu = MAX_Q_MSG;
     if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
 	return -1;
