@@ -10,6 +10,7 @@
 #include <linux/atmsvc.h>
 #include <linux/atmdev.h>
 
+#include "atm.h"
 #include "atmd.h"
 #include "q2931.h"
 #include "qlib.h"
@@ -24,6 +25,8 @@
 
 extern int vci;
 
+static struct sockaddr_atmsvc local; /* local address */
+
 
 static int send_setup(SOCKET *sock)
 {
@@ -49,17 +52,31 @@ static int send_setup(SOCKET *sock)
 	q_assign(&dsc,QF_fw_max_sdu,sock->remote->sas_txtp.max_sdu);
     if (sock->remote->sas_rxtp.max_sdu)
 	q_assign(&dsc,QF_bw_max_sdu,sock->remote->sas_rxtp.max_sdu);
-    q_assign(&dsc,QF_fw_pcr_01,1000); /* Fore doesn't like ATM_OC3_PCR */
-    q_assign(&dsc,QF_bw_pcr_01,1000); /* Fore doesn't like ATM_OC3_PCR */
-    q_assign(&dsc,QF_best_effort,0);
-    q_assign(&dsc,QF_bearer_class,3); /* BCOB-C */
+    switch (sock->remote->sas_txtp.class) {
+	case ATM_UBR:
+    	    q_assign(&dsc,QF_fw_pcr_01,ATM_OC3_PCR);
+    	    q_assign(&dsc,QF_bw_pcr_01,ATM_OC3_PCR);
+   	    q_assign(&dsc,QF_best_effort,0);
+	    break;
+	case ATM_CBR:
+    	    q_assign(&dsc,QF_fw_pcr_01,sock->remote->sas_txtp.min_pcr);
+    	    q_assign(&dsc,QF_bw_pcr_01,sock->remote->sas_rxtp.min_pcr);
+	    break;
+	default:
+	    return -EINVAL;
+    }
+    q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
+    q_assign(&dsc,QF_traf_type,0); /* force presence - UNI 3.0 wants this */
     q_assign(&dsc,QF_upcc,0); /* p2p */
     q_assign(&dsc,QF_qos_fw,0); /* QOS 0 */
     q_assign(&dsc,QF_qos_bw,0);
-    if (sock->local && *sock->local->sas_addr.prv) { /* improve @@@ */
+    if ((sock->local && *sock->local->sas_addr.prv) || (local.sas_family &&
+      *local.sas_addr.prv)) { /* handle E.164 too */
 	q_assign(&dsc,QF_cgpn_plan,2);
 	q_assign(&dsc,QF_cgpn_type,0);
-	q_write(&dsc,QF_cgpn,(void *) sock->local->sas_addr.prv,ATM_ESA_LEN);
+	q_write(&dsc,QF_cgpn,sock->local && *sock->local->sas_addr.prv ?
+	  (void *) sock->local->sas_addr.prv : (void *) local.sas_addr.prv,
+	  ATM_ESA_LEN);
     }
     bhli = &sock->remote->sas_addr.bhli;
     if (bhli->hl_type != ATM_HL_NONE) {
@@ -204,6 +221,8 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		    send_kernel(msg->vcc,0,as_close,-EINVAL,NULL);
 		    return;
 		}
+		sock->pvc.sap_txtp = sock->remote->sas_txtp;
+		sock->pvc.sap_rxtp = sock->remote->sas_rxtp;
 		sock->state = ss_connecting;
 		error = send_setup(sock);
 		if (error) {
@@ -316,12 +335,34 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 }
 
 
+static void itf_notify(int itf,struct sockaddr_atmsvc *addr)
+{
+    char buf[MAX_ATM_ADDR_LEN+1];
+
+    if (!addr->sas_family) {
+	local.sas_family = 0;
+	diag(COMPONENT,DIAG_INFO,"Removed local ATM address");
+	return;
+    }
+    if (atm2text(buf,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) addr,A2T_PRETTY) <
+      0) diag(COMPONENT,DIAG_ERROR,"bad address from kernel");
+    else {
+	diag(COMPONENT,DIAG_INFO,"Local ATM address set to %s",buf);
+	local = *addr;
+    }
+}
+
+
 void from_kernel(struct atmsvc_msg *msg,int size)
 {
     SOCKET *curr;
     struct atm_blli **this;
     int bllis,i;
 
+    if (msg->type == as_itf_notify) {
+	itf_notify(msg->pvc.sap_addr.itf,&msg->local);
+	return;
+    }
     for (curr = sockets; curr; curr = curr->next)
 	if (msg->vcc == curr->id) break;
     if (!curr)
