@@ -14,6 +14,7 @@
 #include <string.h>
 #include <assert.h>
 
+#include "atm.h"
 #include "atmd.h"
 #include "q2931.h"
 #include "qlib.h"
@@ -44,6 +45,7 @@ static void send_call_proceeding(SOCKET *sock)
     if (net) {
 	q_assign(&dsc,QF_vpi,0);
 	q_assign(&dsc,QF_vci,vci); /* @@@ */
+	sock->pvc.sap_addr.itf = 0; /* @@@ use VPCI */
 	sock->pvc.sap_addr.vpi = 0;
 	sock->pvc.sap_addr.vci = vci++;
     }
@@ -77,10 +79,18 @@ static struct sockaddr_atmsvc *build_sap(void)
 	if (q_fetch(&in_dsc,QF_aal_type) != 5)
 	    diag(COMPONENT,DIAG_ERROR,"AAL type %d requested",
 	      q_fetch(&in_dsc,QF_aal_type));
-    if (q_present(&in_dsc,QF_fw_pcr_01))
-	sap->sas_txtp.max_pcr = q_fetch(&in_dsc,QF_fw_pcr_01);
-    if (q_present(&in_dsc,QF_bw_pcr_01))
-	sap->sas_rxtp.max_pcr = q_fetch(&in_dsc,QF_bw_pcr_01);
+    if (q_present(&in_dsc,QF_best_effort))
+	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_UBR;
+    else {
+	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_CBR;
+	if (q_present(&in_dsc,QF_fw_pcr_01))
+	    sap->sas_txtp.min_pcr = sap->sas_txtp.max_pcr =
+	      q_fetch(&in_dsc,QF_fw_pcr_01);
+	if (q_present(&in_dsc,QF_bw_pcr_01))
+	    sap->sas_rxtp.min_pcr = sap->sas_rxtp.max_pcr =
+	      q_fetch(&in_dsc,QF_bw_pcr_01);
+	/* SHOULD ... fail call if anything is missing ... @@@ */
+    }
     if (q_present(&in_dsc,QF_fw_max_sdu))
 	sap->sas_txtp.max_sdu = q_fetch(&in_dsc,QF_fw_max_sdu);
     if (q_present(&in_dsc,QF_bw_max_sdu))
@@ -168,24 +178,35 @@ static void setup_call(unsigned long call_ref)
     this->q2931_state = qs_in_proc;
     this->call_ref = call_ref;
     send_call_proceeding(this);
-    this->local = alloc_sap(sock->local);
+    this->local = sap;
+    this->pvc.sap_txtp = sap->sas_txtp;
+    this->pvc.sap_rxtp = sap->sas_rxtp;
+    /* if (sock->local) *this->local->sas_addr = sock->local->sas_addr; ??? */
     printf("AAL type %ld\n",q_fetch(&in_dsc,QF_aal_type));
     len = q_read(&in_dsc,QF_cdpn_esa,(void *) &buf,sizeof(buf));
     printf("Addr len %d:",len);
     for (i = 0; i < (len < 30 ? len : 30); i++) printf(" %02X",buf[i]);
     putchar('\n');
     if (q_present(&in_dsc,QF_vci)) {
+	this->pvc.sap_addr.itf = 0; /* @@@ use VPCI */
 	this->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
 	this->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
 printf("VPI/VCI: %d/%d\n",this->pvc.sap_addr.vpi,this->pvc.sap_addr.vci);
     }
-    if (q_present(&in_dsc,QF_cgpn)) { /* improve @@@ */
+    if (q_present(&in_dsc,QF_cgpn)) { /* should handle E.164 too */
+	char buffer[MAX_ATM_ADDR_LEN+1];
+
 	this->remote = alloc_t(struct sockaddr_atmsvc);
 	memset(this->remote,0,sizeof(struct sockaddr_atmsvc));
 	this->remote->sas_family = AF_ATMSVC;
 	i = q_read(&in_dsc,QF_cgpn,(void *) this->remote->sas_addr.prv,
 	  ATM_ESA_LEN);
-	printf("addr_len = %d\n",i);
+	this->remote->sas_txtp = sap->sas_txtp;
+	this->remote->sas_rxtp = sap->sas_rxtp;
+	/*printf("addr_len = %d\n",i);*/
+	if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) this->remote,
+	  A2T_PRETTY) < 0) strcpy(buffer,"<invalid address>");
+	diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
     }
     send_kernel(0,sock->id,as_indicate,0,this);
     for (walk = &sock->listen; *walk; walk = &(*walk)->listen);
@@ -267,6 +288,7 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    /* check for 2nd CALL_PROC @@@ */
 	    STOP_TIMER(sock);
 	    if (q_present(&in_dsc,QG_conn_id)) {
+		sock->pvc.sap_addr.itf = 0; /* @@@ should use VPCI */
 		sock->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
 		sock->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
 printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
@@ -282,6 +304,7 @@ printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 	    if (sock->state != ss_connecting) break;
 	    STOP_TIMER(sock);
 	    if (q_present(&in_dsc,QG_conn_id)) {
+		sock->pvc.sap_addr.itf = 0; /* @@@ should use VPCI */
 		sock->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
 		sock->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
 printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
