@@ -11,6 +11,14 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <linux/atm.h>
+#include <atm.h>
+
+
+static void usage(const char *name)
+{
+    fprintf(stderr,"usage: %s [itf.]vpi.vci data [ offset ] \n",name);
+    exit(1);
+}
 
 
 int main(int argc,char **argv)
@@ -18,26 +26,24 @@ int main(int argc,char **argv)
     struct sockaddr_atmpvc addr;
     int s,size,offset;
 
-    if (argc != 5 && argc != 6) {
-	fprintf(stderr,"usage: %s itf vpi vci data [ offset ] \n",argv[0]);
-	return 1;
-    }
+    if (argc != 3 && argc != 4) usage(argv[0]);
     if ((s = socket(PF_ATMPVC,SOCK_DGRAM,ATM_AAL5)) < 0) {
 	perror("socket");
 	return 1;
     }
     memset(&addr,0,sizeof(addr));
-    addr.sap_family = AF_ATMPVC;
-    addr.sap_addr.itf = atoi(argv[1]);
-    addr.sap_addr.vpi = atoi(argv[2]);
-    addr.sap_addr.vci = atoi(argv[3]);
+    if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
+	usage(argv[0]);
     addr.sap_txtp.class = ATM_UBR;
-    offset = argc == 5 ? 0 : atoi(argv[5]);
+    addr.sap_txtp.max_sdu = strlen(argv[2])-offset;
+    offset = argc == 3 ? 0 : atoi(argv[3]);
     if (connect(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
 	return 1;
     }
-    size = write(s,argv[4]+offset,strlen(argv[4])-offset);
-    printf("%d(%s)\n",size,strerror(errno));
+    size = write(s,argv[2]+offset,strlen(argv[2])-offset);
+    printf("%d",size);
+    if (size < 0) printf("(%s)",strerror(errno));
+    putchar('\n');
     return 0;
 }
